@@ -34,7 +34,6 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	char line[MAXPATHLEN];
 	char *p, *user=NULL;
 	extern int remote_version;
-	extern int am_client;
 	extern int am_sender;
 
 	if (*path == '/') {
@@ -52,8 +51,6 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	if (!user) user = getenv("USER");
 	if (!user) user = getenv("LOGNAME");
 
-	am_client = 1;
-
 	fd = open_socket_out(host, rsync_port);
 	if (fd == -1) {
 		exit_cleanup(RERR_SOCKETIO);
@@ -138,16 +135,16 @@ static int rsync_module(int fd, int i)
 		return -1;
 	}
 
-	if (!claim_connection(lp_lock_file(), lp_max_connections())) {
+	if (!claim_connection(lp_lock_file(i), lp_max_connections(i))) {
 		if (errno) {
 			rprintf(FERROR,"failed to open lock file %s : %s\n",
-				lp_lock_file(), strerror(errno));
+				lp_lock_file(i), strerror(errno));
 			io_printf(fd,"@ERROR: failed to open lock file %s : %s\n",
-				  lp_lock_file(), strerror(errno));
+				  lp_lock_file(i), strerror(errno));
 		} else {
 			rprintf(FERROR,"max connections (%d) reached\n",
-				lp_max_connections());
-			io_printf(fd,"@ERROR: max connections (%d) reached - try again later\n", lp_max_connections());
+				lp_max_connections(i));
+			io_printf(fd,"@ERROR: max connections (%d) reached - try again later\n", lp_max_connections(i));
 		}
 		return -1;
 	}
@@ -167,24 +164,28 @@ static int rsync_module(int fd, int i)
 	if (lp_read_only(i))
 		read_only = 1;
 
-	p = lp_uid(i);
-	if (!name_to_uid(p, &uid)) {
-		if (!isdigit(*p)) {
-			rprintf(FERROR,"Invalid uid %s\n", p);
-			io_printf(fd,"@ERROR: invalid uid\n");
-			return -1;
-		} 
-		uid = atoi(p);
-	}
+	am_root = (getuid() == 0);
 
-	p = lp_gid(i);
-	if (!name_to_gid(p, &gid)) {
-		if (!isdigit(*p)) {
-			rprintf(FERROR,"Invalid gid %s\n", p);
-			io_printf(fd,"@ERROR: invalid gid\n");
-			return -1;
-		} 
-		gid = atoi(p);
+	if (am_root) {
+		p = lp_uid(i);
+		if (!name_to_uid(p, &uid)) {
+			if (!isdigit(*p)) {
+				rprintf(FERROR,"Invalid uid %s\n", p);
+				io_printf(fd,"@ERROR: invalid uid\n");
+				return -1;
+			} 
+			uid = atoi(p);
+		}
+
+		p = lp_gid(i);
+		if (!name_to_gid(p, &gid)) {
+			if (!isdigit(*p)) {
+				rprintf(FERROR,"Invalid gid %s\n", p);
+				io_printf(fd,"@ERROR: invalid gid\n");
+				return -1;
+			} 
+			gid = atoi(p);
+		}
 	}
 
 	p = lp_include_from(i);
@@ -208,24 +209,12 @@ static int rsync_module(int fd, int i)
 			return -1;
 		}
 
-		if (chdir("/")) {
+		if (!push_dir("/", 0)) {
 			rprintf(FERROR,"chdir %s failed\n", lp_path(i));
 			io_printf(fd,"@ERROR: chdir failed\n");
 			return -1;
 		}
 
-		if (setgid(gid) || getgid() != gid) {
-			rprintf(FERROR,"setgid %d failed\n", gid);
-			io_printf(fd,"@ERROR: setgid failed\n");
-			return -1;
-		}
-
-		if (setuid(uid) || getuid() != uid) {
-			rprintf(FERROR,"setuid %d failed\n", uid);
-			io_printf(fd,"@ERROR: setuid failed\n");
-			return -1;
-		}
-
 	} else {
 		if (!push_dir(lp_path(i), 0)) {
 			rprintf(FERROR,"chdir %s failed\n", lp_path(i));
@@ -234,7 +223,21 @@ static int rsync_module(int fd, int i)
 		}
 	}
 
-	am_root = (getuid() == 0);
+	if (am_root) {
+		if (setgid(gid)) {
+			rprintf(FERROR,"setgid %d failed\n", gid);
+			io_printf(fd,"@ERROR: setgid failed\n");
+			return -1;
+		}
+
+		if (setuid(uid)) {
+			rprintf(FERROR,"setuid %d failed\n", uid);
+			io_printf(fd,"@ERROR: setuid failed\n");
+			return -1;
+		}
+
+		am_root = (getuid() == 0);
+	}
 
 	io_printf(fd,"@RSYNCD: OK\n");
 
@@ -286,7 +289,7 @@ static int rsync_module(int fd, int i)
 		}
 	}
 
-	ret = parse_arguments(argc, argv);
+	ret = parse_arguments(argc, argv, 0);
 
 	if (request) {
 		if (*auth_user) {
@@ -414,13 +417,9 @@ static int start_daemon(int fd)
 int daemon_main(void)
 {
 	extern char *config_file;
+	extern int orig_umask;
 	char *pid_file;
 
-	/* this ensures that we don't call getcwd after the chroot,
-           which doesn't work on platforms that use popen("pwd","r")
-           for getcwd */
-	push_dir("/", 0);
-
 	if (is_a_socket(STDIN_FILENO)) {
 		int i;
 
@@ -449,16 +448,19 @@ int daemon_main(void)
 	rprintf(FINFO,"rsyncd version %s starting\n",VERSION);
 
 	if (((pid_file = lp_pid_file()) != NULL) && (*pid_file != '\0')) {
-		FILE *f;
+		char pidbuf[16];
+		int fd;
 		int pid = (int) getpid();
 		cleanup_set_pid(pid);
-		if ((f = fopen(lp_pid_file(), "w")) == NULL) {
+		if ((fd = do_open(lp_pid_file(), O_WRONLY|O_CREAT|O_TRUNC,
+					0666 & ~orig_umask)) == -1) {
 		    cleanup_set_pid(0);
 		    fprintf(stderr,"failed to create pid file %s\n", pid_file);
 		    exit_cleanup(RERR_FILEIO);
 		}
-		fprintf(f, "%d\n", pid);
-		fclose(f);
+		slprintf(pidbuf, sizeof(pidbuf), "%d\n", pid);
+		write(fd, pidbuf, strlen(pidbuf));
+		close(fd);
 	}
 
 	start_accept_loop(rsync_port, start_daemon);
