@@ -23,6 +23,7 @@
 #include "rsync.h"
 
 extern int verbose;
+extern int delete_mode;
 
 static struct exclude_struct **exclude_list;
 
@@ -41,7 +42,7 @@ int send_included_file_names(int f,struct file_list *flist)
 	int n;
 	char *p;
 
-	if (!only_included_files || (exclude_the_rest == NULL))
+	if (!only_included_files || (exclude_the_rest == NULL) || delete_mode)
 		return 0;
 
 	if (verbose > 1) {
@@ -60,7 +61,10 @@ int send_included_file_names(int f,struct file_list *flist)
 			/* skip the allowed beginning slashes */
 			p++;
 		}
-		send_file_name(f,flist,p,0,0);
+		/* silently skip files that don't exist to
+		   be more like non-optimized case */
+		if (access(p,0) == 0)
+			send_file_name(f,flist,p,0,0);
 	}
 	exclude_list = ex_list;
 	
@@ -97,6 +101,7 @@ static struct exclude_struct *make_exclude(char *pattern, int include)
 		    only_included_files = 0;
 	    }
 	    ret->regular_exp = 1;
+	    ret->fnmatch_flags = strstr(pattern, "**") ? 0 : FNM_PATHNAME;
 	} else if (!ret->include) {
 		only_included_files = 0;
 	}
@@ -140,7 +145,7 @@ static int check_one_exclude(char *name,struct exclude_struct *ex,
 	}
 
 	if (ex->regular_exp) {
-		if (fnmatch(pattern, name, 0) == 0)
+		if (fnmatch(pattern, name, ex->fnmatch_flags) == 0)
 			return 1;
 	} else {
 		int l1 = strlen(name);
@@ -234,7 +239,12 @@ struct exclude_struct **make_exclude_list(char *fname,
 		int l = strlen(line);
 		if (l && line[l-1] == '\n') l--;
 		line[l] = 0;
-		if (line[0]) add_exclude_list(line,&list,include);
+		if (line[0] && (line[0] != ';') && (line[0] != '#')) {
+			/* Skip lines starting with semicolon or pound.
+			   It probably wouldn't cause any harm to not skip
+			     them but there's no need to save them. */
+			add_exclude_list(line,&list,include);
+		}
 	}
 	fclose(f);
 	return list;
@@ -293,14 +303,62 @@ void recv_exclude_list(int f)
 	}
 }
 
+/* Get the next include/exclude arg from the string. It works in a similar way
+** to strtok - initially an arg is sent over, from then on NULL. This
+** routine takes into account any +/- in the strings and does not
+** consider the space following it as a delimeter.
+*/
+char *get_exclude_tok(char *p)
+{
+	static char *s;
+	static int more;
+	char *t;
 
+	if (p) {
+		s=p;
+		if (*p)
+			more=1;
+	}
+
+	if (!more)
+		return(NULL);
+
+	/* Skip over any initial spaces */
+	while(isspace(*s))
+		s++;
+
+	/* Are we at the end of the string? */
+	if (*s) {
+		/* remember the beginning of the token */
+		t=s;
+
+		/* Is this a '+' or '-' followed by a space (not whitespace)? */
+		if ((*s=='+' || *s=='-') && *(s+1)==' ')
+			s+=2;
+	
+		/* Skip to the next space or the end of the string */
+		while(!isspace(*s) && *s!='\0')
+			s++;
+	} else {
+		t=NULL;
+	}
+
+	/* Have we reached the end of the string? */
+	if (*s)
+		*s++='\0';
+	else
+		more=0;
+	return(t);
+}
+
+	
 void add_exclude_line(char *p)
 {
 	char *tok;
 	if (!p || !*p) return;
 	p = strdup(p);
 	if (!p) out_of_memory("add_exclude_line");
-	for (tok=strtok(p," "); tok; tok=strtok(NULL," "))
+ 	for (tok=get_exclude_tok(p); tok; tok=get_exclude_tok(NULL))
 		add_exclude(tok, 0);
 	free(p);
 }
@@ -311,7 +369,7 @@ void add_include_line(char *p)
 	if (!p || !*p) return;
 	p = strdup(p);
 	if (!p) out_of_memory("add_include_line");
-	for (tok=strtok(p," "); tok; tok=strtok(NULL," "))
+	for (tok=get_exclude_tok(p); tok; tok=get_exclude_tok(NULL))
 		add_exclude(tok, 1);
 	free(p);
 }
