@@ -44,9 +44,12 @@ extern int preserve_gid;
 extern int preserve_times;
 extern int relative_paths;
 extern int copy_links;
+extern int copy_unsafe_links;
 extern int remote_version;
 extern int io_error;
 
+static char topsrcname[MAXPATHLEN];
+
 static struct exclude_struct **local_exclude_list;
 
 static void clean_flist(struct file_list *flist, int strip_root);
@@ -85,6 +88,32 @@ static void list_file_entry(struct file_struct *f)
 }
 
 
+int readlink_stat(const char *Path, STRUCT_STAT *Buffer, char *Linkbuf) 
+{
+#if SUPPORT_LINKS
+	if (copy_links) {
+		return do_stat(Path, Buffer);
+	}
+	if (do_lstat(Path, Buffer) == -1) {
+		return -1;
+	}
+	if (S_ISLNK(Buffer->st_mode)) {
+		int l;
+		if ((l = readlink(Path,Linkbuf,MAXPATHLEN-1)) == -1) {
+			return -1;
+		}
+		Linkbuf[l] = 0;
+		if (copy_unsafe_links && (topsrcname[0] != '\0') &&
+				    unsafe_symlink(Linkbuf, topsrcname)) {
+			return do_stat(Path, Buffer);
+		}
+	}
+	return 0;
+#else
+	return do_stat(Path, Buffer);
+#endif
+}
+
 int link_stat(const char *Path, STRUCT_STAT *Buffer) 
 {
 #if SUPPORT_LINKS
@@ -366,13 +395,15 @@ static int skip_filesystem(char *fname, STRUCT_STAT *st)
 	return (st2.st_dev != filesystem_dev);
 }
 
-static struct file_struct *make_file(char *fname)
+static struct file_struct *make_file(int f, char *fname)
 {
 	struct file_struct *file;
 	STRUCT_STAT st;
 	char sum[SUM_LENGTH];
 	char *p;
 	char cleaned_name[MAXPATHLEN];
+	char linkbuf[MAXPATHLEN];
+	extern int delete_excluded;
 
 	strlcpy(cleaned_name, fname, MAXPATHLEN);
 	cleaned_name[MAXPATHLEN-1] = 0;
@@ -381,7 +412,7 @@ static struct file_struct *make_file(char *fname)
 
 	memset(sum,0,SUM_LENGTH);
 
-	if (link_stat(fname,&st) != 0) {
+	if (readlink_stat(fname,&st,linkbuf) != 0) {
 		io_error = 1;
 		rprintf(FERROR,"%s: %s\n",
 			fname,strerror(errno));
@@ -398,11 +429,12 @@ static struct file_struct *make_file(char *fname)
 			return NULL;
 	}
 	
-	if (!match_file_name(fname,&st))
+	/* f is set to -1 when calculating deletion file list */
+	if (((f != -1) || !delete_excluded) && !match_file_name(fname,&st))
 		return NULL;
 	
 	if (verbose > 2)
-		rprintf(FINFO,"make_file(%s)\n",fname);
+		rprintf(FINFO,"make_file(%d,%s)\n",f,fname);
 	
 	file = (struct file_struct *)malloc(sizeof(*file));
 	if (!file) out_of_memory("make_file");
@@ -437,16 +469,7 @@ static struct file_struct *make_file(char *fname)
 
 #if SUPPORT_LINKS
 	if (S_ISLNK(st.st_mode)) {
-		int l;
-		char lnk[MAXPATHLEN];
-		if ((l=readlink(fname,lnk,MAXPATHLEN-1)) == -1) {
-			io_error=1;
-			rprintf(FERROR,"readlink %s : %s\n",
-				fname,strerror(errno));
-			return NULL;
-		}
-		lnk[l] = 0;
-		file->link = strdup(lnk);
+		file->link = strdup(linkbuf);
 	}
 #endif
 
@@ -488,7 +511,7 @@ void send_file_name(int f,struct file_list *flist,char *fname,
 {
   struct file_struct *file;
 
-  file = make_file(fname);
+  file = make_file(f,fname);
 
   if (!file) return;  
   
@@ -609,14 +632,22 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 	}
 
 	for (i=0;i<argc;i++) {
-		char fname2[MAXPATHLEN];
-		char *fname = fname2;
+		char *fname = topsrcname;
 
 		strlcpy(fname,argv[i],MAXPATHLEN);
 
 		l = strlen(fname);
 		if (l != 1 && fname[l-1] == '/') {
-			strlcat(fname,".",MAXPATHLEN);
+			if ((l == 2) && (fname[0] == '.')) {
+				/*  Turn ./ into just . rather than ./.
+				    This was put in to avoid a problem with
+				      rsync -aR --delete from ./
+				    The send_file_name() below of ./ was
+				    mysteriously preventing deletes */
+				fname[1] = 0;
+			} else {
+				strlcat(fname,".",MAXPATHLEN);
+			}
 		}
 
 		if (link_stat(fname,&st) != 0) {
@@ -652,10 +683,17 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 				*p = '/';
 				for (p=fname+1; (p=strchr(p,'/')); p++) {
 					int copy_links_saved = copy_links;
+					int recurse_saved = recurse;
 					*p = 0;
-					copy_links = 0;
+					copy_links = copy_unsafe_links;
+					/* set recurse to 1 to prevent make_file
+					   from ignoring directory, but still
+					   turn off the recursive parameter to
+					   send_file_name */
+					recurse = 1;
 					send_file_name(f, flist, fname, 0, 0);
 					copy_links = copy_links_saved;
+					recurse = recurse_saved;
 					*p = '/';
 				}
 			} else {
@@ -695,6 +733,8 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 		}
 	}
 
+	topsrcname[0] = '\0';
+
 	if (f != -1) {
 		send_file_entry(NULL,f,0);
 	}
