@@ -17,18 +17,22 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#define False 0
+#define True 1
+
 #define BLOCK_SIZE 700
 #define RSYNC_RSH_ENV "RSYNC_RSH"
 
 #define RSYNC_NAME "rsync"
 #define RSYNCD_CONF "/etc/rsyncd.conf"
 
+#define DEFAULT_LOCK_FILE "/var/run/rsyncd.lock"
 #define URL_PREFIX "rsync://"
 
 #define BACKUP_SUFFIX "~"
 
 /* a non-zero CHAR_OFFSET makes the rolling sum stronger, but is
-   imcompatible with older versions :-( */
+   incompatible with older versions :-( */
 #define CHAR_OFFSET 0
 
 
@@ -43,7 +47,7 @@
 #define SAME_TIME (1<<7)
 
 /* update this if you make incompatible changes */
-#define PROTOCOL_VERSION 19
+#define PROTOCOL_VERSION 20
 #define MIN_PROTOCOL_VERSION 11
 #define MAX_PROTOCOL_VERSION 30
 
@@ -52,7 +56,7 @@
 #define SPARSE_WRITE_SIZE (1024)
 #define WRITE_SIZE (32*1024)
 #define CHUNK_SIZE (32*1024)
-#define MAX_MAP_SIZE (1*1024*1024)
+#define MAX_MAP_SIZE (256*1024)
 #define IO_BUFFER_SIZE (4092)
 #define MAX_READ_BUFFER (1024*1024)
 
@@ -148,11 +152,6 @@
 #endif
 #include <errno.h>
 
-#if defined(HAVE_MMAP) && defined(HAVE_MUNMAP)
-#include <sys/mman.h>
-#define USE_MMAP 1
-#endif
-
 #ifdef HAVE_UTIME_H
 #include <utime.h>
 #endif
@@ -274,6 +273,10 @@
 #define MAX(a,b) ((a)>(b)?(a):(b))
 #endif
 
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 256
+#endif
+
 /* the length of the md4 checksum */
 #define MD4_SUM_LENGTH 16
 #define SUM_LENGTH 16
@@ -303,6 +306,7 @@ struct file_struct {
 	char *sum;
 };
 
+
 struct file_list {
 	int count;
 	int malloced;
@@ -326,15 +330,16 @@ struct sum_struct {
 };
 
 struct map_struct {
-	char *map,*p;
+	char *p;
 	int fd,p_size,p_len;
-	OFF_T size, p_offset;
+	OFF_T file_size, p_offset, p_fd_offset;
 };
 
 struct exclude_struct {
 	char *orig;
 	char *pattern;
 	int regular_exp;
+	int fnmatch_flags;
 	int include;
 	int directory;
 	int local;
@@ -460,6 +465,10 @@ extern int errno;
 #ifndef ACCESSPERMS
 #define ACCESSPERMS 0777
 #endif
+/* Initial mask on permissions given to temporary files.  Mask off setuid
+     bits and group access because of potential race-condition security
+     holes, and mask other access because mode 707 is bizarre */
+#define INITACCESSPERMS 0700
 
 /* handler for null strings in printf format */
 #define NS(s) ((s)?(s):"<NULL>")
